#include <WiFi.h>
#include <HTTPClient.h>
#include <ArduinoJson.h>
#include <Adafruit_NeoPixel.h>
#include "config.h"

HTTPClient http;
StaticJsonDocument<1000> jsonDoc;
Adafruit_NeoPixel leds = Adafruit_NeoPixel(NUM_LEDS, LED_PIN, NEO_GRB + NEO_KHZ800);
long nextUpdate = 0;

void setup()
{
  leds.begin();
  resetLeds();

  Serial.begin(115200);
  while (!Serial);
}

void loop()
{
  if (lmillis() - nextUpdate >= 0) {
    updateWeatherDisplay();
    nextUpdate = lmillis() + CHECK_INTERVAL * 60 * 1000;
  }
}

void updateWeatherDisplay()
{
  connectWiFi();

  Serial.print("Requesting openweather.org...");
  http.begin(weatherUrl);

  int httpCode = http.GET();
  if (httpCode = 200) {
    Serial.println("[ok]");
    String payload = http.getString();

    DeserializationError jsonError = deserializeJson(jsonDoc, payload);
    if (jsonError) {
      Serial.print("deserializeJson() failed: ");
      Serial.println(jsonError.c_str());
    } else {
      const char* city = jsonDoc["name"];
      const char* weather = jsonDoc["weather"][0]["main"];
      const char* weatherDesc = jsonDoc["weather"][0]["description"];
      int weatherId = jsonDoc["weather"][0]["id"];
      double temperatur = jsonDoc["main"]["temp"];
      int humidity = jsonDoc["main"]["humidity"];

      Serial.println("Location:    " + String(city));
      Serial.println("Weather:     " + String(weather) + " (#" + weatherId + ") / " + String(weatherDesc));
      Serial.println("Temperature: " + String(temperatur) + "°C");
      Serial.println("Humidity:    " + String(humidity) + "%");

      showWeather(weatherId);
      showTemperature(temperatur);
      showHumidity(humidity);
    }
  } else {
    Serial.println("[error] " + String(httpCode));
  }

  http.end();
  WiFi.disconnect();
  Serial.println("WiFi disconnected.");
}

void resetLeds()
{
  for (byte i = 0; i < NUM_LEDS; i++) {
    leds.setPixelColor(i, leds.Color(0, 0, 0));
  }
  leds.show();
}

void showWeather(int weatherId)
{
  int ledIndex = 0;
  for (byte i = 0; i < WEATHER_CONDITION_AMOUNT; i++) {
    if (weatherConditions[i].id == weatherId) {
      leds.setPixelColor(ledIndex, weatherConditions[i].red, weatherConditions[i].green, weatherConditions[i].blue);
      leds.show();
      break;
    }
  }
}

void showTemperature(double temperatur)
{
  int ledIndex = 1;
  if (temperatur < -10.0) {
    leds.setPixelColor(ledIndex, 0, 75, 100);
  } else if (temperatur <= 0.0) {
    leds.setPixelColor(ledIndex, 0, 100, 75);
  } else if (temperatur <= 10.0) {
    leds.setPixelColor(ledIndex, 15, 75, 40);
  } else if (temperatur <= 20.0) {
    leds.setPixelColor(ledIndex, 25, 150, 0);
  } else if (temperatur <= 26.0) {
    leds.setPixelColor(ledIndex, 75, 100, 0);
  } else if (temperatur <= 30.0) {
    leds.setPixelColor(ledIndex, 100, 75, 0);
  } else if (temperatur <= 35.0) {
    leds.setPixelColor(ledIndex, 150, 30, 0);
  } else if (temperatur > 35.0) {
    leds.setPixelColor(ledIndex, 150, 0, 20);
  } else {
    leds.setPixelColor(ledIndex, 0, 0, 0);
  }
  leds.show();
}

void showHumidity(int humidity)
{
  int ledIndex = 2;
  if (humidity < 25) {
    leds.setPixelColor(ledIndex, 100, 75, 0);
  } else if (humidity < 50) {
    leds.setPixelColor(ledIndex, 75, 100, 0);
  } else if (humidity < 75) {
    leds.setPixelColor(ledIndex, 30, 120, 0);
  } else if (humidity >= 75) {
    leds.setPixelColor(ledIndex, 0, 150, 35);
  } else {
    leds.setPixelColor(ledIndex, 0, 0, 0);
  }
  leds.show();
}

void connectWiFi()
{
  Serial.print(F("Connecting WiFi..."));
  WiFi.begin(wifiSSID, wifiPassword);
  for (byte i = 0; i < 30; i++) {
    Serial.print(F("."));
    if (WiFi.status() == WL_CONNECTED) {
      break;
    }
    delay(1000);
  }
  Serial.println(F("[ok]"));
}

